/* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

/** @file
* @brief Example template project.
* @defgroup nrf_templates_example Example Template
*
*/

#include <stdbool.h>
#include <stdint.h>

#include "nrf.h"
#include "nrf_gpio.h"
#include "nordic_common.h"
#include "nrf_delay.h"
#include "app_error.h"
#include "app_timer.h"
#include "app_util_platform.h"
#include "boards.h"
#include "bsp.h"

#include "app_uart.h"

#define MAX_TEST_DATA_BYTES                     (15U)                /**< max number of test bytes to be used for tx and rx. */
#define UART_TX_BUF_SIZE                        256                         /**< UART TX buffer size. */
#define UART_RX_BUF_SIZE                        1                           /**< UART RX buffer size. */

#define ADC_REF_VOLTAGE_IN_MILLIVOLTS           1200
#define ADC_PRE_SCALING_COMPENSATION            3 
#define ADC_RESULT_IN_MILLI_VOLTS(ADC_VALUE)\
				((((ADC_VALUE) * ADC_REF_VOLTAGE_IN_MILLIVOLTS) / 1023) * ADC_PRE_SCALING_COMPENSATION)

uint16_t m_vdd_measurement_volts;
uint16_t m_vdd_measurement_raw;
// uint8_t FirstHalf;
// uint8_t SecondHalf;

void ADC_IRQHandler(void)
{
    
	if(NRF_ADC->EVENTS_END)
	{
		/* acknowledge event */
		NRF_ADC->EVENTS_END = 0;

		/* read battery voltage */
		m_vdd_measurement_raw = (uint16_t)NRF_ADC->RESULT;
		m_vdd_measurement_volts = ADC_RESULT_IN_MILLI_VOLTS(m_vdd_measurement_raw);
       printf("VDD Measurement: %d mV\n",m_vdd_measurement_volts);
		NRF_ADC->TASKS_STOP = 1;
		NRF_ADC->ENABLE = 0;
	}
}

void adc_init(void)
{

	/* setup ADC for capturing battery voltage */
	NRF_ADC->CONFIG = (
		(ADC_CONFIG_REFSEL_VBG                      << ADC_CONFIG_REFSEL_Pos) |
		(ADC_CONFIG_INPSEL_SupplyOneThirdPrescaling << ADC_CONFIG_INPSEL_Pos) |
		(ADC_CONFIG_RES_10bit                        << ADC_CONFIG_RES_Pos)
	);
	NRF_ADC->INTENSET = (
		(ADC_INTENSET_END_Enabled << ADC_INTENSET_END_Pos)
	);
    NVIC_SetPriority(ADC_IRQn, APP_IRQ_PRIORITY_LOW);
	NVIC_EnableIRQ(ADC_IRQn);
}

void adc_start(void)
{
	/* start ADC voltage conversion */
	NRF_ADC->ENABLE = 1;
	NRF_ADC->TASKS_START = 1;
}

void uart_error_handle(app_uart_evt_t * p_event)
{
    if (p_event->evt_type == APP_UART_COMMUNICATION_ERROR)
    {
        APP_ERROR_HANDLER(p_event->data.error_communication);
    }
    else if (p_event->evt_type == APP_UART_FIFO_ERROR)
    {
        APP_ERROR_HANDLER(p_event->data.error_code);
    }
}

void uart_init()
{
 uint32_t err_code;
    const app_uart_comm_params_t comm_params =
      {
          RX_PIN_NUMBER,
          TX_PIN_NUMBER,
          RTS_PIN_NUMBER,
          CTS_PIN_NUMBER,
          APP_UART_FLOW_CONTROL_DISABLED,
          false,
          UART_BAUDRATE_BAUDRATE_Baud9600
      };

    APP_UART_FIFO_INIT(&comm_params,
                         UART_RX_BUF_SIZE,
                         UART_TX_BUF_SIZE,
                         uart_error_handle,
                         APP_IRQ_PRIORITY_LOW,
                         err_code);

    APP_ERROR_CHECK(err_code);
}
/**
 * @brief Function for application main entry.
 */
int main(void)
{
    adc_init();
    uart_init();
    while (true)
    {
        adc_start();
        nrf_delay_ms(1000);
        // Do nothing.
    }
}
/** @} */
